import React from "react";
import './highlight.css'
const markTagOffset = 13;
export default class Highlight extends React.Component {
    constructor(props) {
        super(props);
        this.historyRanges = [];
        this.state={}
    }

    onMouseUpHandler = (index) => {
        const selectionObj = (window.getSelection && window.getSelection());
        if (selectionObj.isCollapsed) return;
        const selection = selectionObj.toString();
        const anchorOffset = selectionObj.anchorOffset;
        const focusOffset = selectionObj.focusOffset;
        const anchorNode = selectionObj.anchorNode;
        const focusNode = selectionObj.focusNode;
        const position = anchorNode.compareDocumentPosition(focusNode);
        let forward = false;

        if (position === anchorNode.DOCUMENT_POSITION_FOLLOWING) {
            forward = true;
        } else if (position === 0) {
            forward = (focusOffset - anchorOffset) > 0;
        }
        let endOffset = 0;
        let startOffset = 0;
        let focusFound = false;
        let anchorFound = false;
        let parentNode = anchorNode.parentNode;
        let focusParentNode = focusNode.parentNode;
        if (anchorNode.parentElement.nodeName === "MARK") {
            parentNode = parentNode.parentNode;
        }
        if (focusNode.parentElement.nodeName === "MARK") {
            focusParentNode = focusParentNode.parentNode;
        }
        if (parentNode != focusParentNode) return;

        for (let i = 0; i < parentNode.childNodes.length; i++) {
            if (focusFound) break;
            let node = parentNode.childNodes[i];
            if (node.nodeName === 'MARK') {
                node = node.childNodes[0]
            }
            if (node !== focusNode) {
                endOffset += node.nodeType === 1 ? node.innerText.length : node.length;
            }
            if (node === focusNode) {
                endOffset += focusOffset;
                focusFound = true
            }
        }

        for (let j = 0; j < parentNode.childNodes.length; j++) {
            if (anchorFound) break;
            let node = parentNode.childNodes[j];
            if (node.nodeName == 'MARK') {
                node = node.childNodes[0]
            }
            if (node != anchorNode) {
                startOffset += node.nodeType == 1 ? node.innerText.length : node.length;
            }
            if (node == anchorNode) {
                startOffset += anchorOffset;
                anchorFound = true
            }
        }
        let selectionStart = startOffset;
        let selectionEnd = endOffset;
        if (!forward) {
            selectionStart = endOffset;
            selectionEnd = startOffset;
        }

        this.selectionHandler(selection, selectionStart, selectionEnd - 1, index)
    }

    selectionHandler = (selection, selectionStart, selectionEnd, paragraphIndex) => {
        let p = this.state.paragraphs ? [...this.state.paragraphs] : [...this.props.paragraphs];
        let paragraphString = this.props.paragraphs[paragraphIndex];
        this.mergeRanges({start: selectionStart, end: selectionEnd}, paragraphIndex);
        this.historyRanges[paragraphIndex].forEach((range, index) => {
            let start = range['start'] + (index * markTagOffset)
            let end = range['end'] + (index * markTagOffset)
            paragraphString = `${paragraphString.slice(0, start)}<mark>${paragraphString.slice(start, end + 1)}</mark>${paragraphString.slice(end + 1, paragraphString.length)}`;
        })
        p[paragraphIndex] = paragraphString;
        this.setState({paragraphs: p})
    }


    mergeRanges = (range, paraIndex) => {
        if (this.historyRanges[paraIndex] == null) {
            this.historyRanges[paraIndex] = [];
        }
        let mid = Math.floor((this.historyRanges[paraIndex].length) / 2);
        let index = this.historyRanges[paraIndex].length > 0 ? this.binarySearch(0, this.historyRanges[paraIndex].length - 1, mid, range, paraIndex) : mid;
        let merged = false;
        for (var i = 0; i < this.historyRanges[paraIndex].length; i++) {
            let merge = false;
            if (merged) {
                merge = this.mergeifMergeable(this.historyRanges[paraIndex][i], this.historyRanges[paraIndex][i - 1])
            } else {
                merge = this.mergeifMergeable(this.historyRanges[paraIndex][i], range);
            }
            if (merge) {
                this.historyRanges[paraIndex][i] = merge;
                merged = true;
            }
        }

        if (!merged) {
            this.historyRanges[paraIndex] = [...this.historyRanges[paraIndex].slice(0, index), range, ...this.historyRanges[paraIndex].slice(index, this.historyRanges[paraIndex].length)];
        }

    }

    mergeifMergeable = (source, target) => {
        let exp = (source['start'] <= target['start'] && target['start'] <= source['end'] ? 1 :
            target['end'] <= source['end'] && target['end'] >= source['start'] ? 2 :
                target['start'] <= source['start'] && target['end'] >= source['end'] ? 3 : 0);
        if (exp === 1) {
            return {start: source['start'], end: Math.max(target['end'], source['end'])}
        } else if (exp === 2) {
            return {start: Math.min(source['start'], target['start']), end: source['end']}
        } else if (exp === 3) {
            return target;
        }
        return false;
    }

    binarySearch = (start, end, mid, newEle, paraIndex) => {

        while (start < end) {
            let ele = this.historyRanges[paraIndex][mid];
            if (ele['start'] === newEle['start']) {
                return mid;
            } else if (ele['start'] > newEle['start']) {
                end = mid - 1
                if (end < start) break;
                mid = Math.ceil((start + end) / 2);
            } else {
                start = mid + 1;
                if (start > end) break;
                mid = Math.ceil((start + end) / 2);
            }
        }
        if (newEle['start'] > this.historyRanges[paraIndex][mid]['start']) {
            return mid + 1;
        } else {
            return mid;
        }
    }

    render() {
        let p = this.state.paragraphs || this.props.paragraphs;
        return(
            <>
                {
                    p.map((paragraph,i) => {
                        return <p dangerouslySetInnerHTML={{__html: paragraph}}
                                  key={i} className={`${this.props.className}`} id="" onMouseUp={(e) => {
                            e.preventDefault();
                            this.onMouseUpHandler(i)
                        }}/>
                    })
                }
            </>
        )
    }

}